@@ -1,6 +1,6 @@
 package Mobile::Wurfl;
 
-$VERSION = '1.07';
+$VERSION = '1.08';
 
 use strict;
 use warnings;
@@ -8,6 +8,7 @@ use DBI;
 use DBD::mysql;
 use XML::Parser( Style => "Object" );
 require LWP::UserAgent;
+use HTTP::Date;
 use Template;
 use File::Spec;
 use File::Basename;
@@ -23,7 +24,7 @@ sub new
 {
     my $class = shift;
     my %opts = (
-        wurfl_home => ".",
+        wurfl_home => "/tmp",
         db_descriptor => "DBI:mysql:database=wurfl:host=localhost", 
         db_username => 'wurfl',
         db_password => 'wurfl',
@@ -37,17 +38,17 @@ sub new
     my $self = bless \%opts, $class;
     if ( ! $self->{verbose} )
     {
-        open( LOG, ">" . File::Spec->devnull() )
+        open( STDERR, ">" . File::Spec->devnull() )
     }
     elsif ( $self->{verbose} == 1 )
     {
-        open( LOG, ">$self->{wurfl_home}/wurfl.log" );
+        open( STDERR, ">$self->{wurfl_home}/wurfl.log" );
     }
     else
     {
-        open( LOG, ">&STDERR" );
+        warn "log to STDERR\n";
     }
-    print LOG "connecting to $self->{db_descriptor} as $self->{db_username}\n";
+    print STDERR "connecting to $self->{db_descriptor} as $self->{db_username}\n";
     $self->{dbh} ||= DBI->connect( 
         $self->{db_descriptor},
         $self->{db_username},
@@ -64,14 +65,15 @@ sub new
     return $self;
 }
 
+
 sub _tables_exist
 {
     my $self = shift;
-    my @db_tables = $self->{dbh}->tables();
-    my %db_tables = map { $_ => 1 } $self->{dbh}->tables();
-    for ( keys %tables )
+    my %db_tables = map { my $key = $_ =~ /(.*)\.(.*)/ ? $2 : $_ ; $key => 1 } $self->{dbh}->tables();
+    for my $table ( keys %tables )
     {
-        return 0 unless $db_tables{"`$_`"};
+        return 0 unless $db_tables{$self->{dbh}->quote_identifier($table)};
+
     }
     return 1;
 }
@@ -84,8 +86,9 @@ sub _init
     {
         die "tables don't exist on $self->{db_descriptor}: try running $self->create_tables()\n";
     }
+
     $self->{last_update_sth} = $self->{dbh}->prepare( 
-        "SELECT UNIX_TIMESTAMP( ts ) FROM $self->{device_table_name} ORDER BY ts DESC LIMIT 1"
+        "SELECT ts FROM $self->{device_table_name} ORDER BY ts DESC LIMIT 1"
     );
     $self->{user_agents_sth} = $self->{dbh}->prepare( 
         "SELECT DISTINCT user_agent FROM $self->{device_table_name}" 
@@ -149,7 +152,6 @@ sub create_tables
 {
     my $self = shift;
     my $sql = shift;
-    return if $self->_tables_exist();
     unless ( $sql )
     {
         my $tt = Template->new();
@@ -159,7 +161,7 @@ sub create_tables
     for my $statement ( split( /\s*;\s*/, $sql ) )
     {
         next unless $statement =~ /\S/;
-        $self->{dbh}->do( $statement );
+        $self->{dbh}->do( $statement ) or die "$statement failed\n";
     }
 }
 
@@ -169,7 +171,7 @@ sub touch( $$ )
     my $time = shift;
     die "no path" unless $path;
     die "no time" unless $time;
-    print LOG "touch $path ($time)\n";
+    print STDERR "touch $path ($time)\n";
     return utime( $time, $time, $path );
 }
 
@@ -178,9 +180,9 @@ sub last_update
     my $self = shift;
     $self->_init();
     $self->{last_update_sth}->execute();
-    my ( $ts ) = $self->{last_update_sth}->fetchrow();
+    my ( $ts ) = str2time($self->{last_update_sth}->fetchrow());
     $ts ||= 0;
-    print LOG "last update: $ts\n";
+    print STDERR "last update: $ts\n";
     return $ts;
 }
 
@@ -192,14 +194,16 @@ sub rebuild_tables
     my $last_update = $self->last_update();
     if ( $local <= $last_update )
     {
-        print LOG "$self->{wurfl_file} has not changed since the last database update\n";
+        print STDERR "$self->{wurfl_file} has not changed since the last database update\n";
         return 0;
     }
-    print LOG "$self->{wurfl_file} is newer than the last database update\n";
-    print LOG "flush dB tables ...\n";
+    print STDERR "$self->{wurfl_file} is newer than the last database update\n";
+    print STDERR "flush dB tables ...\n";
+    $self->{dbh}->begin_work;
     $self->{dbh}->do( "DELETE FROM $self->{device_table_name}" );
     $self->{dbh}->do( "DELETE FROM $self->{capability_table_name}" );
     my ( $device_id, $group_id );
+    print STDERR "create XML parser ...\n";
     my $xp = new XML::Parser(
         Handlers => {
             Start => sub { 
@@ -229,17 +233,21 @@ sub rebuild_tables
             },
         }
     );
+    print STDERR "parse XML ...\n";
     $xp->parsefile( $self->{wurfl_file} );
+    print STDERR "commit dB ...\n";
+    $self->{dbh}->commit;
     return 1;
 }
 
 sub update
 {
     my $self = shift;
+    print STDERR "get wurfl\n";
     my $got_wurfl = $self->get_wurfl();
-    print LOG "got wurfl: $got_wurfl\n";
+    print STDERR "got wurfl: $got_wurfl\n";
     my $rebuilt ||= $self->rebuild_tables();
-    print LOG "rebuilt: $rebuilt\n";
+    print STDERR "rebuilt: $rebuilt\n";
     return $got_wurfl || $rebuilt;
 }
 
@@ -247,22 +255,22 @@ sub get_local_stats
 {
     my $self = shift;
     return ( 0, 0 ) unless -e $self->{wurfl_file};
-    print LOG "stat $self->{wurfl_file} ...\n";
+    print STDERR "stat $self->{wurfl_file} ...\n";
     my @stat = ( stat $self->{wurfl_file} )[ 7,9 ];
-    print LOG "@stat\n";
+    print STDERR "@stat\n";
     return @stat;
 }
 
 sub get_remote_stats
 {
     my $self = shift;
-    print LOG "HEAD $self->{wurfl_url} ...\n";
+    print STDERR "HEAD $self->{wurfl_url} ...\n";
     my $response = $self->{ua}->head( $self->{wurfl_url} );
     die $response->status_line unless $response->is_success;
     die "can't get content_length\n" unless $response->content_length;
     die "can't get last_modified\n" unless $response->last_modified;
     my @stat = ( $response->content_length, $response->last_modified );
-    print LOG "@stat\n";
+    print STDERR "@stat\n";
     return @stat;
 }
 
@@ -274,11 +282,11 @@ sub get_wurfl
  
     if ( $local[1] == $remote[1] )
     {
-        print LOG "@local and @remote are the same\n";
+        print STDERR "@local and @remote are the same\n";
         return 0;
     }
-    print LOG "@local and @remote are different\n";
-    print LOG "GET $self->{wurfl_url} -> $self->{wurfl_file} ...\n";
+    print STDERR "@local and @remote are different\n";
+    print STDERR "GET $self->{wurfl_url} -> $self->{wurfl_file} ...\n";
 
     #create a temp filename
     my $tempfile = "$self->{wurfl_home}/wurfl_$$";
@@ -377,14 +385,14 @@ sub canonical_ua
     my $deviceid = $self->{deviceid_sth}->fetchrow;
     if ( $deviceid )
     {
-        print LOG "$ua found\n";
+        print STDERR "$ua found\n";
         return $ua;
     }
     $ua = substr( $ua, 0, -1 );
     # $ua =~ s/^(.+)\/(.*)$/$1\// ;
     unless ( length $ua )
     {
-        print LOG "can't find canonical user agent\n";
+        print STDERR "can't find canonical user agent\n";
         return;
     }
     return $self->canonical_ua( $ua );
@@ -397,7 +405,7 @@ sub device
     $self->_init();
     $self->{device_sth}->execute( $deviceid );
     my $device = $self->{device_sth}->fetchrow_hashref;
-    print LOG "can't find device for user deviceid $deviceid\n" unless $device;
+    print STDERR "can't find device for user deviceid $deviceid\n" unless $device;
     return $device;
 }
 
@@ -408,7 +416,7 @@ sub deviceid
     $self->_init();
     $self->{deviceid_sth}->execute( $ua );
     my $deviceid = $self->{deviceid_sth}->fetchrow;
-    print LOG "can't find device id for user agent $ua\n" unless $deviceid;
+    print STDERR "can't find device id for user agent $ua\n" unless $deviceid;
     return $deviceid;
 }
 
@@ -440,19 +448,19 @@ sub lookup_value
 sub cleanup
 {
     my $self = shift;
-    print LOG "cleanup ...\n";
+    print STDERR "cleanup ...\n";
     if ( $self->{dbh} )
     {
-        print LOG "drop tables\n";
+        print STDERR "drop tables\n";
         for ( keys %tables )
         {
-            print LOG "DROP TABLE IF EXISTS $_\n";
+            print STDERR "DROP TABLE IF EXISTS $_\n";
             $self->{dbh}->do( "DROP TABLE IF EXISTS $_" );
         }
     }
     return unless $self->{wurfl_file};
     return unless -e $self->{wurfl_file};
-    print LOG "unlink $self->{wurfl_file}\n";
+    print STDERR "unlink $self->{wurfl_file}\n";
     unlink $self->{wurfl_file} || die "Can't remove $self->{wurfl_file}: $!\n";
 }
 
@@ -541,7 +549,7 @@ The list of possible options are as follows:
 
 =item wurfl_home
 
-Used to set the default home diretory for Mobile::Wurfl. This is where the cached copy of the wurfl.xml file is stored. It defaults to current directory.
+Used to set the default home diretory for Mobile::Wurfl. This is where the cached copy of the wurfl.xml file is stored. It defaults to /tmp.
 
 =item db_descriptor
 
